(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['exports', 'react'], factory);
    } else if (typeof exports !== "undefined") {
        factory(exports, require('react'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod.exports, global.react);
        global.FocusOverlay = mod.exports;
    }
})(this, function (exports, _react) {
    'use strict';

    Object.defineProperty(exports, "__esModule", {
        value: true
    });
    exports.getOverlayStyles = undefined;

    var _react2 = _interopRequireDefault(_react);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    function _classCallCheck(instance, Constructor) {
        if (!(instance instanceof Constructor)) {
            throw new TypeError("Cannot call a class as a function");
        }
    }

    var _createClass = function () {
        function defineProperties(target, props) {
            for (var i = 0; i < props.length; i++) {
                var descriptor = props[i];
                descriptor.enumerable = descriptor.enumerable || false;
                descriptor.configurable = true;
                if ("value" in descriptor) descriptor.writable = true;
                Object.defineProperty(target, descriptor.key, descriptor);
            }
        }

        return function (Constructor, protoProps, staticProps) {
            if (protoProps) defineProperties(Constructor.prototype, protoProps);
            if (staticProps) defineProperties(Constructor, staticProps);
            return Constructor;
        };
    }();

    function _possibleConstructorReturn(self, call) {
        if (!self) {
            throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
        }

        return call && (typeof call === "object" || typeof call === "function") ? call : self;
    }

    function _inherits(subClass, superClass) {
        if (typeof superClass !== "function" && superClass !== null) {
            throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
        }

        subClass.prototype = Object.create(superClass && superClass.prototype, {
            constructor: {
                value: subClass,
                enumerable: false,
                writable: true,
                configurable: true
            }
        });
        if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
    }

    var _extends = Object.assign || function (target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];

            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }

        return target;
    };

    var MAX_OPACITY = 0.8;
    var TRANSITION_SPEED = 0.3;

    var getOverlayStyles = exports.getOverlayStyles = function getOverlayStyles(x, y, width, height) {
        var shared = {
            position: 'absolute',
            backgroundColor: '#000',
            transition: 'all ' + TRANSITION_SPEED + 's linear'
        };

        var styles = {
            top: _extends({}, shared),
            bottom: _extends({}, shared),
            left: _extends({}, shared),
            right: _extends({}, shared)
        };
        if (typeof x === 'number') {
            styles.top.top = '0';
            styles.top.left = '0';
            styles.top.right = '0';
            styles.top.height = y + 'px';
            styles.bottom.top = y + height + 'px', styles.bottom.bottom = '0';
            styles.bottom.left = '0';
            styles.bottom.right = '0';
            styles.left.top = y + 'px';
            styles.left.left = '0';
            styles.left.width = x + 'px';
            styles.left.height = height + 'px';
            styles.right.top = y + 'px';
            styles.right.left = x + width + 'px';
            styles.right.right = '0';
            styles.right.height = height + 'px';
        }
        return styles;
    };

    var FocusOverlay = function (_React$Component) {
        _inherits(FocusOverlay, _React$Component);

        function FocusOverlay(props) {
            _classCallCheck(this, FocusOverlay);

            var _this = _possibleConstructorReturn(this, (FocusOverlay.__proto__ || Object.getPrototypeOf(FocusOverlay)).call(this, props));

            _this.state = {
                opacity: 0,
                visibility: 'hidden'
            };
            return _this;
        }

        _createClass(FocusOverlay, [{
            key: 'transitionStyles',
            value: function transitionStyles(opacity, x, y, width, height) {
                var _this2 = this;

                var newState = void 0;

                if (x) {
                    //if any coordinates have been passed then try set all the coordinates
                    newState = { x: x, y: y, width: width, height: height, opacity: opacity };
                } else {
                    //otherwise ignore all but opacity.
                    newState = { opacity: opacity };
                }

                if (opacity === 0) {
                    //if the target state is hidden, the we need to set the visibility to hidden after the opacity transition
                    setTimeout(function () {
                        _this2.setState({
                            visibility: 'hidden',
                            x: undefined, y: undefined, width: undefined, height: undefined
                        });
                    }, TRANSITION_SPEED * 1000);
                } else {
                    newState.visibility = 'visible';
                }
                setTimeout(function () {
                    _this2.setState(newState);
                });
            }
        }, {
            key: 'tryMove',
            value: function tryMove(focusComponent) {
                var _focusComponent$curre = focusComponent.current.getBoundingClientRect(),
                    x = _focusComponent$curre.x,
                    y = _focusComponent$curre.y,
                    width = _focusComponent$curre.width,
                    height = _focusComponent$curre.height;

                if (this.state.x !== x || this.state.y !== y || this.state.width !== width || this.state.height !== height || this.state.opacity !== MAX_OPACITY) {
                    this.transitionStyles(MAX_OPACITY, x, y, width, height);
                }
            }
        }, {
            key: 'render',
            value: function render() {
                var _props = this.props,
                    focusComponent = _props.focusComponent,
                    onExit = _props.onExit;

                var styles = void 0;
                if (!focusComponent) {
                    if (this.state.opacity !== 0) {
                        this.transitionStyles(0);
                    }
                } else {
                    this.tryMove(focusComponent);
                }

                var _state = this.state,
                    x = _state.x,
                    y = _state.y,
                    width = _state.width,
                    height = _state.height,
                    opacity = _state.opacity,
                    visibility = _state.visibility;

                styles = getOverlayStyles(x, y, width, height);

                var containerStyle = {
                    opacity: opacity,
                    transition: 'opacity ' + TRANSITION_SPEED + 's linear',
                    visibility: visibility
                };

                return _react2.default.createElement(
                    'div',
                    { style: containerStyle, 'data-selector': 'focus-overlay' },
                    _react2.default.createElement('div', { style: styles.top, onClick: onExit }),
                    _react2.default.createElement('div', { style: styles.left, onClick: onExit }),
                    _react2.default.createElement('div', { style: styles.right, onClick: onExit }),
                    _react2.default.createElement('div', { style: styles.bottom, onClick: onExit })
                );
            }
        }]);

        return FocusOverlay;
    }(_react2.default.Component);

    ;

    exports.default = FocusOverlay;
});